USE AdventureWorks
GO
SET NOCOUNT ON

-- Look at Actual Execution Plan

DECLARE @TempTable TABLE
(ProductID int PRIMARY KEY,
 ProductNumber nvarchar(50),
 SumTotal numeric(38,6),
 SumDiscount numeric(38,6))


DECLARE @StartTime DATETIME
SET @StartTime = GETDATE()

INSERT INTO @TempTable (ProductID, ProductNumber)
SELECT ProductID, ProductNumber
FROM Production.Product p

UPDATE tt
SET SumTotal = (SELECT SUM(LineTotal) SumTotal FROM Sales.SalesOrderDetail sd WHERE sd.ProductID = tt.ProductID ),
	SumDiscount = (SELECT SUM(LineTotal*UnitPriceDiscount) SumDiscount FROM Sales.SalesOrderDetail sd WHERE sd.ProductID = tt.ProductID )
FROM @TempTable tt

print 'Table Variable Duration with correlated subqueries: ' + CAST(DATEDIFF(ms, @StartTime, getdate()) as varchar(10)) + ' ms'

DELETE @TempTable
SET @StartTime = GETDATE()

INSERT INTO @TempTable (ProductID, ProductNumber)
SELECT ProductID, ProductNumber
FROM Production.Product p

UPDATE tt
SET SumTotal = SalesTotals.SumTotal,
	SumDiscount = SalesTotals.SumDiscount
FROM @TempTable tt
INNER JOIN (
 			    SELECT ProductID, SUM(LineTotal) SumTotal, SUM(LineTotal*UnitPriceDiscount) SumDiscount 
				FROM Sales.SalesOrderDetail 
				GROUP BY ProductID
				HAVING SUM(LineTotal*UnitPriceDiscount) > SUM(LineTotal)*0.01 
			) SalesTotals on tt.ProductID = SalesTotals.ProductID

print 'Table Variable Duration with derived table: ' + CAST(DATEDIFF(ms, @StartTime, getdate()) as varchar(10)) + ' ms'


GO

CREATE TABLE #TempTable
(ProductID int PRIMARY KEY,
 ProductNumber nvarchar(50),
 SumTotal numeric(38,6),
 SumDiscount numeric(38,6))


DECLARE @StartTime DATETIME
SET @StartTime = GETDATE()

INSERT INTO #TempTable (ProductID, ProductNumber)
SELECT ProductID, ProductNumber
FROM Production.Product p

UPDATE tt
SET SumTotal = (SELECT SUM(LineTotal) SumTotal FROM Sales.SalesOrderDetail sd WHERE sd.ProductID = tt.ProductID ),
	SumDiscount = (SELECT SUM(LineTotal*UnitPriceDiscount) SumDiscount FROM Sales.SalesOrderDetail sd WHERE sd.ProductID = tt.ProductID )
FROM #TempTable tt

print 'Temp Table Duration with correlated subqueries: ' + CAST(DATEDIFF(ms, @StartTime, getdate()) as varchar(10)) + ' ms'

TRUNCATE TABLE #TempTable
SET @StartTime = GETDATE()

INSERT INTO #TempTable (ProductID, ProductNumber)
SELECT ProductID, ProductNumber
FROM Production.Product p

UPDATE tt
SET SumTotal = SalesTotals.SumTotal,
	SumDiscount = SalesTotals.SumDiscount
FROM #TempTable tt
INNER JOIN (
			    SELECT ProductID, SUM(LineTotal) SumTotal, SUM(LineTotal*UnitPriceDiscount) SumDiscount 
				FROM Sales.SalesOrderDetail 
				GROUP BY ProductID
				HAVING SUM(LineTotal*UnitPriceDiscount) > SUM(LineTotal)*0.01 
			) SalesTotals on tt.ProductID = SalesTotals.ProductID

print 'Temp Table Duration with derived table: ' + CAST(DATEDIFF(ms, @StartTime, getdate()) as varchar(10)) + ' ms'

DROP TABLE #TempTable